

; ==============================================================================
; Blitz3D GeoWall Game Engine Template
; (C) 2004 Jason Leigh, Electronic Visualization Laboratory
;
; Use this to get started on your Blitz3D GeoWall Game
; This code shows:
;	1. how to set up GeoWall stereo
;	2. how to create a game loop that will keep constant update rate on CPUs with different speeds
;	3. loads a simple animated test object
;
; Keys:
;	1 = mono
;	2 = stereo
;	3 = reverse stereo
;	+ = increase stereo separation
;	- = decrease sterep separation
;
; Created: 7/15/2004
; Version: 8/17/2004
; For comments email: spiff@evl.uic.edu
; -----------------------------------------------------------------------------




; Include geowall stereo library
Include "BlitzGeoWallStereoLib.bb"


; ==============================================================================
; Init graphics and stereo cameras
Global g_screenXSize = 1024
Global g_screenYSize = 768

Print "Choose Display Mode:"
Print "1 - Standard GeoWall Stereo (2048x768)"
Print "2 - Desktop (1280x1024)"
Print "3 - Desktop (1600x1200)"
Print "4 - Desktop (1024x768)"

choice=WaitKey()

Select choice

Case 49
	g_screenXSize=2048
	g_screenYSize=768

Case 50
	g_screenXSize=1280
	g_screenYSize=1024

Case 51
	g_screenXSize=1600
	g_screenYSize=1200

Default
	g_screenXSize=1024
	g_screenYSize=768
End Select


Graphics3D g_screenXSize, g_screenYSize

; Eye separation (-ve means cross-eyed stereo) 
Global g_eyeSeparation#=0.1
Global g_reverseStereo#=1
SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)

; Stereo viewing flag: 0=stereo 1=mono
Global g_viewMode=0
SetBuffer BackBuffer() 

;camera=SL_GetRightCamera()
;CameraRange camera,0.1,500000
;camera=SL_GetLeftCamera()
;CameraRange camera,0.1,500000

;PositionEntity g_mainCamera,0,0.5,-1.5


Type Player
	Field entity,camera,microphone
	Field ctrl_mode,cam_mode,cam_dir,ignition
	Field pitch#,yaw#,pitch_speed#,yaw_speed#,roll#,thrust#
	Field cam_pitch#,cam_yaw#
	Field screen_width,screen_height
End Type

Type Node
	Field x, y, z, nextn
End Type

Dim nodelist.Node(3)

nodelist(1) = New Node
nodelist(2) = New Node
nodelist(3) = New Node
nodelist(1)\x = 0
nodelist(1)\y = 50
nodelist(1)\z = -2500
nodelist(1)\nextn = 2
nodelist(2)\x = 50
nodelist(2)\y = 50
nodelist(2)\z = -2300
nodelist(2)\nextn = 1
;nodelist(3)\x = 50
;nodelist(3)\y = 20
;nodelist(3)\z = -2500
;nodelist(3)\nextn = 1

Global autopilot = 0

Collisions 1,10,2,2	;sphere-to-polygon, colliding collisions

Global terr

CreateScene()

plane=LoadAirPlane( "media/biplane.x" )
; PlayMusic is for loading and playing background music.  Can be used with
; channel commands for control: StopChannel, PauseChannel, ResumeChannel
; Supports raw/mod/s3m/xm/it/mid/rmi/wav/mp2/mp3/ogg/wma/asf
; playmusic unfortunately, DOES NOT SUPPORT LOOPING.
;Global music = PlayMusic( "media/overworld.mp3" )
; LoadSound, PlaySound can achieve similar effects, but only works with
; raw, wav, mp3, & ogg
; one however, needs to be careful not to try using the soundeffect variable
; where a channel is supposed to be used. (leads to memory access violation)
bgmusic_snd = LoadSound("media/overworld.mp3")
LoopSound(bgmusic_snd)
Global music = PlaySound(bgmusic_snd)
Global sndpitch = 44100
Global sndvolume# = 1
Global sndpan# = 0

player1.Player=CreatePlayer( plane,0,0,GraphicsWidth(),GraphicsHeight(),1 )
player1\ctrl_mode = 1;
;player1.Player=CreatePlayer( plane,0,0,640,239,1 )
;player2.Player=CreatePlayer( plane,0,240,640,239,2 )

detail=2000:morph=True
TerrainDetail terr,detail,morph

HidePointer
FlushMouse


Global cube=CreateCube()
EntityColor cube,255,0,0
; ==============================================================================
; Set the game update/calculation rate to 30 frames per second.
; This is NOT the same as FRAME RATE which marks how rapidly the graphics image is updating.
; In this code, the game update/calculation rate is locked down so that the game behaves
; the same for different speed of computers, but the graphics always refreshes as fast as possible.5
Const c_GameRate=30
period=1000/c_GameRate
;theTime=MilliSecs()-period
theTime=MilliSecs()

; ==============================================================================
;
;                      *******  MAIN GAME LOOP  *******
;
; ==============================================================================
; MAIN GAME LOOP that maintains stable frame rate for all parts of the game
;
While Not KeyHit( 1 ) 

	; Compute how much time has elapsed thru the entire main loop
	Repeat
		elapsedTime=MilliSecs()-theTime
	Until elapsedTime<>0
	
	;how many 'frames' have elapsed	
	elapsedFrames=elapsedTime/period
	
	; If more than 1 frame has elapsed then we have to compute
	; where what the state of the world would have been.
	; This is crucial for slower computers.
	; So this loop goes thru each frame of missed time and computes
	; what the new state of the world should be by simplying
	; calling the users UpdateGameState elapsedFrames times.
	For k=1 To elapsedFrames
	
		; When elapsedFrames is reached, save the state of the world so that tweening
		; can be used to get an accurate interpolation of what the state of the world
		; should be.
		If k=elapsedFrames Then CaptureWorld
		theTime=theTime+period
		
		; Update the game's state (e.g. position of each entity)		
		GameProcess()

		; Update all animation properties- e.g. compute collision checks
		UpdateWorld
	Next

	;Compute the tweening value (a number from 0 to 1)
	; 0 means the state of the world when CaptureWorld was called
	; 1 means the current interpolated state of the world
	; e.g. if elapsedTime is 1 second and desired period is 3 second
	; then 1mod3=1 hence tween=1/period = 1/3 
	; so we interpolate 1/3 distance from the captured point and the present time.
	;
	; if elapsedTime = 5 seconds and desired period is 3 seconds;
	; 1 frame of time would have passed so it would have gone thru
	; the for loop once computing the new frame state
	; Since 5mod3=2 hence tween=2/3 so we tween 2/3 the way between the captured state
	; and the current state.
	tween#=Float(elapsedTime Mod period)/Float(period)


	; Switch to mono mode by pressing 1
	If KeyHit(2)>0 Then
	g_viewMode=1
		SL_SwitchToMono()
	EndIf
	
	; Switch to stereo mode by pressing 2
	If KeyHit(3)>0 Then
	g_viewMode=0
		g_reverseStereo#=1
		SL_SwitchToStereo()
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf

	; Switch to cross eye stereo mode by pressing 3
	If KeyHit(4)>0 Then
		g_viewMode=0
		g_reverseStereo#=-1
		SL_SwitchToStereo()
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	
	; Change eye separation by pressing + and -
	If KeyHit(13)>0 Then
		g_eyeSeparation#=g_eyeSeparation#+0.1
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	
	If KeyHit(12) > 0 Then
		g_eyeSeparation#=g_eyeSeparation#-0.1
		If g_eyeSeparation# < 0 Then g_eyeSeparation#=0
		SL_SetEyeSeparation(g_reverseStereo# * g_eyeSeparation#)
	EndIf
	

	; Adjust stereo cameras and render the world
	SL_Update()
	RenderWorld tween#
	

	; Drawing occurs to the backbuffer, so you need to flip the backbuffer
	; to the front to see it. (see explanation in class lecture.)
	Flip


Wend 


; ***********************************************************************
; DO YOUR OWN STUFF HERE!!!!!!
Function GameProcess()
	For p.Player=Each Player
		UpdatePlayer(p)
	Next
End Function

Function UpdatePlayer( p.Player )

	Local x_dir,y_dir,z_dir
	
	; F# keys. (F1-F4)
	If KeyHit(59) p\cam_mode=1
	If KeyHit(60) p\cam_mode=2
	If KeyHit(61) p\cam_mode=3
	If KeyHit(62) p\cam_mode=4
		
	Select p\ctrl_mode
	Case 1:
		If KeyDown(203) x_dir=-1
		If KeyDown(205) x_dir=1
		
		If KeyDown(200) y_dir=-1
		If KeyDown(208) y_dir=1
		
		If KeyDown(30) z_dir=1
		If KeyDown(44) z_dir=-1
		
		If x_dir<0
			p\yaw_speed=p\yaw_speed + (4-p\yaw_speed)*.04
		Else If x_dir>0
			p\yaw_speed=p\yaw_speed + (-4-p\yaw_speed)*.04
		Else
			p\yaw_speed=p\yaw_speed + (-p\yaw_speed)*.02
		EndIf
			
		If y_dir<0
			p\pitch_speed=p\pitch_speed + (2-p\pitch_speed)*.2
		Else If y_dir>0
			p\pitch_speed=p\pitch_speed + (-2-p\pitch_speed)*.2
		Else
			p\pitch_speed=p\pitch_speed + (-p\pitch_speed)*.1
		EndIf
		
		If z_dir>0			;faster?
			p\thrust=p\thrust + (1.5-p\thrust)*.04	;1.5
		Else If z_dir<0		;slower?
			p\thrust=p\thrust + (-p\thrust)*.04
		EndIf
		If KeyHit(15) p\ctrl_mode = 2
	Case 2:
		p\yaw_speed = p\yaw_speed + ((JoyX() * -4) - p\yaw_speed) * .04
		p\pitch_speed=p\pitch_speed + ((JoyY() * -2)-p\pitch_speed)*.2
		If JoyXDir() = 0 p\yaw_speed=p\yaw_speed + (-p\yaw_speed)*.02
		If JoyYDir() = 0 p\pitch_speed=p\pitch_speed + (-p\pitch_speed)*.1
		
		p\thrust = p\thrust + (((JoyU()-1)/2 * -1.5)-p\thrust) * .3
		
		z_dir = 1;
		;If JoyDown(1) z_dir=1
		;If JoyDown(2) z_dir=-1
		
		If KeyHit(15) p\ctrl_mode = 1
	End Select

	; begin misc handling - pathed piloting & sound.
	If KeyHit(25) Then
		If autopilot > 0 Then
			autopilot = 0
		Else
			autopilot = 1
		EndIf
	EndIf
	; using PauseChannel() here.  StopChannel() will not let you restart the sound,
	; if using stopchannel, you need to re-play the sound.
	If KeyHit(19) Then
		; reset things to 'normal'
		ResumeChannel(music)
		sndpitch = 44100
		sndvolume# = 1
		sndpan# = 0
		ChannelPitch(music, sndpitch)
		ChannelVolume(music, sndvolume)
		ChannelPan(music, sndpan)
	EndIf
	If KeyHit(20) PauseChannel(music)
	If KeyHit(21) ResumeChannel(music)
	
	
	If KeyHit(34) Then
		sndpitch = sndpitch - 500
		ChannelPitch(music, sndpitch)
	EndIf
	If KeyHit(35) Then
		sndpitch = sndpitch + 500
		ChannelPitch(music, sndpitch)
	EndIf
		
	If KeyHit(48) Then
		sndpan = sndpan - 0.1
		ChannelPan(music, sndpan)
	EndIf
	If KeyHit(49) Then
		sndpan = sndpan + 0.1
		ChannelPan(music, sndpan)
	EndIf
	
	If KeyHit(22) Then ; 'u'
		sndvolume = sndvolume + 0.1
		ChannelVolume(music, sndvolume)
	EndIf
	If KeyHit(36) Then ; 'j'
		sndvolume = sndvolume - 0.1
		ChannelVolume(music, sndvolume)
	EndIf
	; end sound
	
	p\yaw=p\yaw+p\yaw_speed
	If p\yaw<-180 Then p\yaw=p\yaw+360
	If p\yaw>=180 Then p\yaw=p\yaw-360
	
	p\pitch=p\pitch+p\pitch_speed
	If p\pitch<-180 Then p\pitch=p\pitch+360
	If p\pitch>=180 Then p\pitch=p\pitch-360
		
	p\roll=p\yaw_speed*30
	If autopilot > 0 Then
		;cube=CreateCube()
		PositionEntity cube, nodelist(autopilot)\x, nodelist(autopilot)\y, nodelist(autopilot)\z
		PointEntity p\entity, cube, 0
		If EntityDistance(p\entity, cube) < 2 Then
			autopilot = nodelist(autopilot)\nextn
		EndIf
		;FreeEntity(cube)
	Else 
		RotateEntity p\entity,p\pitch,p\yaw,p\roll
		;see if y/p/r funcs are working...
		t_p#=EntityPitch( p\entity )
		t_y#=EntityYaw( p\entity )
		t_r#=EntityRoll( p\entity )
		RotateEntity p\entity,t_p,t_y,t_r
	EndIf
	
	If p\ignition
		MoveEntity p\entity,0,0,p\thrust
	Else If z_dir>0
		p\ignition=True
	EndIf
	
	
	If p\camera
		Select p\cam_mode
		Case 1:
			EntityParent p\camera,p\entity
			RotateEntity p\camera,0,p\yaw,0,True
			PositionEntity p\camera,EntityX(p\entity),EntityY(p\entity),EntityZ(p\entity),True
			MoveEntity p\camera,0,0.5,-2
			PointEntity p\camera,p\entity,p\roll/2
		Case 2:
			EntityParent p\camera,0
			PositionEntity p\camera,EntityX(p\entity),EntityY(p\entity),EntityZ(p\entity)
			TranslateEntity p\camera,0,1,-5
			PointEntity p\camera,p\entity,0
		Case 3:
			EntityParent p\camera,p\entity
			PositionEntity p\camera,0,.25,0
			RotateEntity p\camera,0,0,0
		Case 4:
			EntityParent p\camera,0
			PointEntity p\camera,p\entity,0
		End Select
		; Mouse x and y speed
		mxs=MouseXSpeed()
		mys=MouseYSpeed()
		
		; Destination camera angle x and y values
		dest_cam_yaw#=dest_cam_yaw#-mxs
		dest_cam_pitch#=dest_cam_pitch#+mys
	
		; Current camera angle x and y values
		p\cam_yaw=p\cam_yaw+((dest_cam_yaw-cam_yaw)/5)
		p\cam_pitch=p\cam_pitch+((dest_cam_pitch-cam_pitch)/5)
		
		RotateEntity p\camera,p\cam_pitch#,p\cam_yaw#,0
		
		; Reset mouse position to centre of screen
		MoveMouse 300,200
	EndIf
	
End Function

Function LoadAirPlane( file$ )
	pivot=CreatePivot()
	plane=LoadMesh( file$,pivot )
	ScaleMesh plane,.125,.25,.125	;make it more spherical!
	RotateEntity plane,0,180,0	;and align to z axis
	EntityRadius pivot,1
	EntityType pivot,1
	HideEntity pivot
	Return pivot
End Function

Function CreatePlayer.Player( plane,vp_x,vp_y,vp_w,vp_h,ctrl_mode )
; This is the main camera that you control in the game.
	p.Player=New Player
	p\ctrl_mode=ctrl_mode
	p\cam_mode=1
	x#=ctrl_mode*10
	z#=ctrl_mode*10-2500
	p\entity=CopyEntity( plane )
	;PositionEntity p\entity,x,TerrainY( terr,x,0,z )+50,z
	PositionEntity p\entity, 20, 20, -2500
	RotateEntity p\entity,0,180,0
	ResetEntity p\entity
	
	;Global g_mainCamera = SL_CreateStereoCameras()
	;SL_SetUpCameras()
	
	p\camera=SL_CreateStereoCameras()
	SL_SetUpCameras()
	PositionEntity p\camera,0,3,-10
	;CameraViewport p\camera,vp_x,vp_y,vp_w,vp_h
	;CameraRange p\camera,0.1,3000
	acamera=SL_GetRightCamera()
	CameraRange acamera,0.1,500000
	CameraClsColor acamera,0,192,255
	CameraFogColor acamera,0,192,255
	CameraFogRange acamera,1000,3000
	bcamera=SL_GetLeftCamera()
	CameraRange bcamera,0.1,500000
	CameraClsColor bcamera,0,192,255
	CameraFogColor bcamera,0,192,255
	CameraFogRange bcamera,1000,3000
;	CameraZoom p\camera,1.5
	If use_fog Then 
		CameraFogMode acamera,1
		CameraFogMode bcamera,1
	EndIf
	;p\microphone=CreateListener(p\camera) ; Create listener, make it child of camera 
	; 0, 0, 0 - very boring, can hear the plane EVERYWHERE.
	;p\microphone = CreateListener(p\camera, 0, 0, 0)
	; 0, 1, 0 - weird effects.
	;p\microphone = CreateListener(p\camera, 0, 1, 0)
	; 2, 2, 2 - high rolloff
	;p\microphone = CreateListener(p\camera, 2, 2, 2)
	; 1, 2, 1 - very boring, can hear the plane EVERYWHERE.
	;p\microphone = CreateListener(p\camera, 1, 2, 1)
	;p\microphone = CreateListener(p\camera, 1, 10, 1)
;	p\microphone = CreateListener(p\camera, 1, 0.1, 1)
	; very fast dropoff
	;p\microphone = CreateListener(p\camera, 400, 500, .1)
	p\microphone = CreateListener(p\camera, 40, 50, .2)
	;p\microphone = CreateListener(p\camera, 1, 1, 1)

	s_propeller = Load3DSound("media/prop.wav")
	LoopSound s_propeller
	EmitSound (s_propeller, p\entity)
	screen_height = GraphicsHeight()
	screen_width = GraphicsWidth()
	Return p
End Function

Function CreateScene()

	;setup lighting
	l=CreateLight()
	RotateEntity l,45,45,0
	AmbientLight 32,32,32
	
	;Load terrain
	terr=LoadTerrain( "media/hmap_1024.bmp" )
	ScaleEntity terr,20,800,20
	PositionEntity terr,-20*512,0,-20*512
	EntityFX terr,1
	EntityType terr,10

	;apply textures to terrain	
	tex1=LoadTexture( "media/coolgrass2.bmp",1 )
	ScaleTexture tex1,10,10
	tex2=LoadTexture( "media/lmap_256.bmp" )
	ScaleTexture tex2,TerrainSize(terr),TerrainSize(terr)
	EntityTexture terr,tex1,0,0
	EntityTexture terr,tex2,0,1
	
	;and ground plane
	plane=CreatePlane()
	ScaleEntity plane,20,1,20
	PositionEntity plane,-20*512,0,-20*512
	EntityTexture plane,tex1,0,0
	EntityOrder plane,3
	EntityFX plane,1
	EntityType plane,10
	
	;create cloud planes
	tex=LoadTexture( "media/cloud_2.bmp",3 )
	ScaleTexture tex,1000,1000
	p=CreatePlane()
	EntityTexture p,tex
	EntityFX p,1
	PositionEntity p,0,450,0
	p=CopyEntity( p )
	RotateEntity p,0,0,180
	
	;create water plane
	tex=LoadTexture( "media/water-2_mip.bmp",3 )
	ScaleTexture tex,10,10
	p=CreatePlane()
	EntityTexture p,tex
	EntityBlend p,1
	EntityAlpha p,.75
	PositionEntity p,0,10,0
	EntityFX p,1
	
End Function


End